/*
Expressmvc on valmis express-sovellusrunko joka noudattaa
MVC-mallia. Kansiossa routes on reititys omaan viewiin ja
ulkomaailmaan. Kansiossa views on sovelluksen backendin viewit.
models ja controllers -kansiot pitää tarvittaessa tehdä itse.
Reiteissä (routes) voi olla kontrolleritoimintoja. Kansiossa bin
on www-serverikoodi.

App.js on Expressin päätiedosto josta sovellus käynnistyy
*/
// tarvittavien pakettien requiret
const createError = require('http-errors');
const express = require('express');
const path = require('path');
const cookieParser = require('cookie-parser');
const logger = require('morgan');
const cors = require('cors');

// reitysfilujen require
const indexRouter = require('./routes/index');
const usersRouter = require('./routes/users');

const app = express(); // luodaan Express app
// cors voimassa kaikille reiteille
app.use(cors());

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'ejs');

// Middlewaren eli Expressin lisäpalikoiden käyttöönottoa

// loggeri loggaa konsoliin tapahtumia
app.use(logger('dev'));
// voidaan jäsentää JSON-dataa
app.use(express.json());
// voidaan jäsentää urlencoded -dataa
app.use(express.urlencoded({ extended: true }));
// voidaan jäsentää dataa cookieista
app.use(cookieParser());
// kansio josta voi servata staattisia filuja
app.use(express.static(path.join(__dirname, 'public')));

// reititysten käyttöönotto
app.use('/', indexRouter);
app.use('/users', usersRouter);

// Periaatteessa reititys voisi sijaita tähän tyyliin app.js -tiedostossa, mutta
// parempi käytänne on toteuttaa reititykset omissa tiedostoissaan routes -kansiossa
app.get('/restapi2', function (req, res) {
  res.json({
    name: 'Urho Kekkonen',
    job: 'President',
    country: 'Finland',
  });
});

// catch 404 and forward to error handler
app.use(function (req, res, next) {
  next(createError(404));
});

// error handler
app.use(function (err, req, res, next) {
  // set locals, only providing error in development
  res.locals.message = err.message;
  res.locals.error = req.app.get('env') === 'development' ? err : {};

  // render the error page
  res.status(err.status || 500);
  res.render('error');
});

module.exports = app;
